@@ -25,7 +25,7 @@ class Arpeggio(abjad_parameters.abc.BangFirstAttachment):
 
     def process_leaf(self, leaf: abjad.Leaf) -> LeafOrLeafSequence:
         direction = self.indicator.direction
-        direction = getattr(self._string_to_direction, direction, direction)
+        direction = self._string_to_direction.get(direction, direction)
         abjad.attach(abjad.Arpeggio(direction=direction), leaf)
         return leaf
 
@@ -45,9 +45,7 @@ class Trill(abjad_parameters.abc.BangFirstAttachment):
 class Cue(abjad_parameters.abc.BangFirstAttachment):
     def process_leaf(self, leaf: abjad.Leaf) -> LeafOrLeafSequence:
         abjad.attach(
-            abjad.Markup(
-                string=rf"\markup \rounded-box {{ {self.indicator.nth_cue} }}"
-            ),
+            abjad.Markup(rf"\markup \rounded-box {{ {self.indicator.index} }}"),
             leaf,
             direction=abjad.enums.UP,
         )
@@ -67,7 +65,7 @@ class WoodwindFingering(abjad_parameters.abc.BangFirstAttachment):
 \\override #'(size . {self.fingering_size})
 {{
     \\woodwind-diagram
-    #'{self.instrument}
+    #'{self.indicator.instrument}
     #'((cc . {self._tuple_to_scheme_list(self.indicator.cc)})
        (lh . {self._tuple_to_scheme_list(self.indicator.left_hand)})
        (rh . {self._tuple_to_scheme_list(self.indicator.right_hand)}))
@@ -85,7 +83,7 @@ class Tremolo(abjad_parameters.abc.BangEachAttachment):
     def process_leaf(self, leaf: abjad.Leaf) -> LeafOrLeafSequence:
         abjad.attach(
             abjad.StemTremolo(
-                self.indicator.n_flags * (2**leaf.written_duration.flag_count)
+                self.indicator.flag_count * (2**leaf.written_duration.flag_count)
             ),
             leaf,
         )
@@ -93,47 +91,39 @@ class Tremolo(abjad_parameters.abc.BangEachAttachment):
 
 
 class ArtificalHarmonic(abjad_parameters.abc.BangEachAttachment):
-    @staticmethod
-    def _change_note_head_style(leaf: abjad.Chord) -> None:
-        abjad.tweak(leaf.note_heads[1]).NoteHead.style = "#'harmonic"
-
     @staticmethod
     def _convert_and_test_leaf(leaf: abjad.Leaf) -> tuple[abjad.Leaf, bool]:
         # return True if artifical_harmonic can be attached and False if
         # artifical harmonic can't be attached
 
-        if isinstance(leaf, abjad.Chord):
-            try:
-                assert len(leaf.note_heads) == 1
-            except AssertionError:
+        match leaf:
+            case abjad.Chord():
+                if len(leaf.note_heads) != 1:
+                    warnings.warn(
+                        "Can't attach artifical harmonic on chord with more or less"
+                        " than one pitch!"
+                    )
+                    return leaf, False
+                return leaf, True
+            case abjad.Note():
+                new_abjad_leaf = abjad.Chord(
+                    [leaf.written_pitch],
+                    leaf.written_duration,
+                )
+                for indicator in abjad.get.indicators(leaf):
+                    if type(indicator) != dict:
+                        abjad.attach(indicator, new_abjad_leaf)
+
+                return new_abjad_leaf, True
+            case _:
                 warnings.warn(
-                    "Can't attach artifical harmonic on chord with more or less"
-                    " than one pitch!"
+                    f"Can't attach artifical harmonic on abjad leaf "
+                    f"'{leaf}' of type '{type(leaf)}'!"
                 )
                 return leaf, False
 
-            return leaf, True
-
-        elif isinstance(leaf, abjad.Note):
-            new_abjad_leaf = abjad.Chord(
-                [leaf.written_pitch],
-                leaf.written_duration,
-            )
-            for indicator in abjad.get.indicators(leaf):
-                if type(indicator) != dict:
-                    abjad.attach(indicator, new_abjad_leaf)
-
-            return new_abjad_leaf, True
-
-        else:
-            warnings.warn(
-                f"Can't attach artifical harmonic on abjad leaf "
-                f"'{leaf}' of type '{type(leaf)}'!"
-            )
-            return leaf, False
-
     def _get_second_pitch(self, abjad_pitch: abjad.Pitch) -> abjad.Pitch:
-        return abjad_pitch + self.indicator.n_semitones
+        return abjad_pitch + self.indicator.semitone_count
 
     def process_leaf(self, leaf: abjad.Leaf) -> LeafOrLeafSequence:
         leaf, is_attachable = ArtificalHarmonic._convert_and_test_leaf(leaf)
@@ -141,16 +131,112 @@ class ArtificalHarmonic(abjad_parameters.abc.BangEachAttachment):
             first_pitch = leaf.note_heads[0].written_pitch
             second_pitch = self._get_second_pitch(first_pitch)
             leaf.written_pitches = abjad.PitchSegment([first_pitch, second_pitch])
-            ArtificalHarmonic._change_note_head_style(leaf)
+            set_note_head_style(leaf, note_head_index=1)
         return leaf
 
 
-class PreciseNaturalHarmonic(abjad_parameters.abc.BangEachAttachment):
-    @staticmethod
-    def _convert_leaf(leaf: abjad.Leaf) -> tuple[abjad.Leaf, bool]:
-        if isinstance(leaf, abjad.Chord):
-            return leaf
+class NaturalHarmonicNodeList(abjad_parameters.abc.AbjadAttachment):
+    replace_leaf_by_leaf = False
 
+    def process_leaf_tuple(
+        self,
+        leaf_tuple: tuple[abjad.Leaf, ...],
+        previous_attachment: typing.Optional[abjad_parameters.abc.AbjadAttachment],
+    ) -> typing.Sequence[abjad.Leaf]:
+        if len((indicator := self.indicator)) > 2:
+            warnings.warn(
+                "Can only represent double harmonics, "
+                "but not triple or more. The following nodes "
+                f"are ignored:\n\n\t{self.indicator[2:]}"
+            )
+            indicator = indicator[:2]
+
+        stem_direction_tuple = self._node_tuple_to_stem_direction_tuple(
+            tuple(indicator)
+        )
+
+        # We put each harmonic into its own voice, because we
+        # don't want to have all harmonics on the same stem,
+        # because then it's not clear for the player which node
+        # on which string she or he needs to play. If the stems
+        # are separated this is very clear.
+        container = abjad.Container([], simultaneous=True)
+        for node, stem_direction in zip(indicator, stem_direction_tuple):
+            voice = abjad.Voice([])
+            for leaf in leaf_tuple:
+                leaf = self._leaf_to_chord(leaf)
+                self._process_leaf(leaf, node, stem_direction)
+                voice.append(leaf)
+            container.append(voice)
+        return container
+
+    def _process_leaf(
+        self,
+        leaf_to_process: abjad.Chord,
+        node: music_parameters.NaturalHarmonic.Node,
+        stem_direction: bool | type[None],
+    ):
+        m2a = self.mutwo_pitch_to_abjad_pitch
+        pitch_segment = [m2a(node.pitch)]
+
+        if self.indicator.write_string:
+            pitch_segment.insert(0, m2a(node.string.tuning_original))
+
+        leaf_to_process.written_pitches = abjad.PitchSegment(pitch_segment)
+
+        if self.indicator.write_string and self.indicator.parenthesize_lower_note_head:
+            leaf_to_process.note_heads[0].is_parenthesized = True
+
+        if self.indicator.harmonic_note_head_style:
+            set_note_head_style(
+                leaf_to_process, note_head_index=int(self.indicator.write_string)
+            )
+
+        if stem_direction in (True, False):
+            abjad.attach(
+                abjad.LilyPondLiteral(
+                    r"\once \override Staff.Stem.thickness = 2"
+                    "\n"
+                    r"\once \override Voice.Stem.direction = "
+                    f"{[-1, 1][int(stem_direction)]}",
+                    site="before",
+                ),
+                leaf_to_process,
+            )
+
+        if stem_direction is False:
+            abjad.attach(
+                abjad.LilyPondLiteral(
+                    r"\once \override NoteColumn #'force-hshift = #1.5", site="before"
+                ),
+                leaf_to_process,
+            )
+
+    @staticmethod
+    def _node_tuple_to_stem_direction_tuple(
+        node_tuple: tuple[music_parameters.NaturalHarmonic.Node]
+        | tuple[
+            music_parameters.NaturalHarmonic.Node, music_parameters.NaturalHarmonic.Node
+        ]
+    ) -> tuple[bool | type[None], ...]:
+        if (node_count := len(node_tuple)) == 2:
+            node0, node1 = node_tuple  # type: ignore
+            if node0.pitch > node1.pitch:
+                stem_direction_tuple = (False, True)
+            elif node0.pitch == node1.pitch:
+                stem_direction_tuple = (True, True)
+            else:
+                stem_direction_tuple = (True, False)
+        elif node_count == 1:
+            stem_direction_tuple = (None,)  # type: ignore
+        else:
+            raise NotImplementedError(node_count)
+        return stem_direction_tuple
+
+    @staticmethod
+    def _leaf_to_chord(leaf: abjad.Leaf) -> abjad.Chord:
+        if isinstance(leaf, abjad.Chord):
+            return abjad.mutate.copy(leaf)
         else:
             new_abjad_leaf = abjad.Chord(
                 "c",
@@ -159,29 +245,8 @@ class PreciseNaturalHarmonic(abjad_parameters.abc.BangEachAttachment):
             for indicator in abjad.get.indicators(leaf):
                 if type(indicator) != dict:
                     abjad.attach(indicator, new_abjad_leaf)
-
             return new_abjad_leaf
 
-    def process_leaf(self, leaf: abjad.Leaf) -> LeafOrLeafSequence:
-        if isinstance(leaf, (abjad.Chord, abjad.Note)):
-            leaf = PreciseNaturalHarmonic._convert_leaf(leaf)
-            leaf.written_pitches = abjad.PitchSegment(
-                [
-                    self.indicator.string_pitch,
-                    abjad.NamedPitch(
-                        self.indicator.string_pitch.name,
-                        octave=self.indicator.octave.number + 1,
-                    ),
-                ]
-            )
-            leaf.note_heads[1]._written_pitch = self.indicator.played_pitch
-            if self.indicator.harmonic_note_head_style:
-                ArtificalHarmonic._change_note_head_style(leaf)
-            if self.indicator.parenthesize_lower_note_head:
-                leaf.note_heads[0].is_parenthesized = True
-
-        return leaf
-
 
 class StringContactPoint(abjad_parameters.abc.ToggleAttachment):
     def __init__(self, *args, **kwargs):
@@ -367,24 +432,12 @@ class BreathMark(
 class Fermata(abjad_parameters.abc.BangFirstAttachment):
     def process_leaf(self, leaf: abjad.Leaf) -> LeafOrLeafSequence:
         abjad.attach(
-            abjad.Fermata(self.indicator.fermata_type),
+            abjad.Fermata(self.indicator.type),
             leaf,
         )
         return leaf
 
 
-class NaturalHarmonic(
-    abjad_parameters.abc.BangFirstAttachment,
-):
-    def process_leaf(self, leaf: abjad.Leaf) -> LeafOrLeafSequence:
-        abjad.attach(
-            abjad.LilyPondLiteral(r"\flageolet", directed="up", site="after"),
-            leaf,
-        )
-
-        return leaf
-
-
 class Prall(
     abjad_parameters.abc.BangFirstAttachment,
 ):
@@ -553,7 +606,7 @@ class Ottava(abjad_parameters.abc.ToggleAttachment):
         previous_attachment: typing.Optional[abjad_parameters.abc.AbjadAttachment],
     ) -> LeafOrLeafSequence:
         abjad.attach(
-            abjad.Ottava(self.indicator.n_octaves, site="before"),
+            abjad.Ottava(self.indicator.octave_count, site="before"),
             leaf,
         )
         return leaf
@@ -564,7 +617,7 @@ class Ottava(abjad_parameters.abc.ToggleAttachment):
         previous_attachment: typing.Optional[abjad_parameters.abc.AbjadAttachment],
     ) -> tuple[abjad.Leaf, ...]:
         # don't attach ottava = 0 at start (this is the default notation)
-        if previous_attachment is not None or self.indicator.n_octaves != 0:
+        if previous_attachment is not None or self.indicator.octave_count != 0:
             return super().process_leaf_tuple(leaf_tuple, previous_attachment)
         else:
             return leaf_tuple
@@ -616,11 +669,12 @@ class Ornamentation(abjad_parameters.abc.BangFirstAttachment):
 
     def _make_markup(self) -> abjad.Markup:
         return abjad.Markup(
+            r"\markup { "
             r"\vspace #-0.25 { \fontsize #-4 { "
-            rf"\rounded-box {{ {self.indicator.n_times} "
-            r"\hspace #-0.4 \path 0.25 "
+            rf"\rounded-box {{ {self.indicator.count} "
+            r"\hspace #-0.4 \path #0.25 "
             f"{self._direction_to_ornamentation_command[self.indicator.direction]}"
-            "}}}"
+            "}}}}"
         )
 
     def process_leaf(self, leaf: abjad.Leaf) -> LeafOrLeafSequence:
@@ -779,6 +833,15 @@ class AfterGraceNoteSequentialEvent(abjad_parameters.abc.BangLastAttachment):
         return leaf
 
 
+def set_note_head_style(
+    leaf: abjad.Chord, note_head_index: int = 0, style: str = "#'harmonic"
+):
+    abjad.tweak(
+        leaf.note_heads[note_head_index],
+        abjad.Tweak(rf"\tweak NoteHead.style {style}"),
+    )
+
+
 # Auto define all due to many classes
 __all__ = tuple(
     name
